package ru.amse.timkina.archiver.ui;

import java.awt.*;
import javax.swing.*;
import javax.swing.plaf.basic.*;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumnModel;
import javax.swing.tree.TreeSelectionModel;

import ru.amse.timkina.archiver.ui.menu.ExitAction;
import ru.amse.timkina.archiver.ui.menu.OpenAction;
import ru.amse.timkina.archiver.ui.menu.SelectionListener;
import ru.amse.timkina.archiver.ui.menu.UnzipAction;
import ru.amse.timkina.archiver.ui.menu.MakeZipAction;
import ru.amse.timkina.archiver.ui.table.MainTable;
import ru.amse.timkina.archiver.ui.table.MainTableModel;
import ru.amse.timkina.archiver.ui.tree.FileTreeModel;
import ru.amse.timkina.archiver.filetree.IDirectory;

public class ArchiverFrame extends JFrame {
	private final MainTableModel myModel;
	private final FileTreeModel myTreeModel;
	
	public ArchiverFrame(IDirectory directory) {
		super("Archiver");
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		myModel = new MainTableModel(directory);	
		myTreeModel = new FileTreeModel(directory);
		MainTable table = new MainTable(myModel);
		JTree tree = new JTree(myTreeModel);
		table.addDirectoryChangeListener(new DirectoryChangeListener(tree));		
		setJMenuBar(createMenu(myModel, myTreeModel, table, this));
		getContentPane().add(createToolBar(myModel, myTreeModel, table), BorderLayout.NORTH);		
	    final JTableHeader header = table.getTableHeader();
		header.setReorderingAllowed(false);
		header.setBorder(new BasicBorders.FieldBorder(Color.gray, Color.darkGray, Color.black, Color.black));
		TableColumnModel columns = table.getColumnModel();
		header.setColumnModel(columns);
		header.setBackground(Color.magenta);			
		tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.addTreeWillExpandListener(new FileTreeWillExpandListener(tree, myModel));
		JSplitPane pane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, 
		    new JScrollPane(tree), new JScrollPane(table));
		pane.setDividerLocation(200);
		pane.setResizeWeight(0.3);		
		getContentPane().add(pane);
		setSize(600, 500);
	}
	
	private JMenuBar createMenu(MainTableModel model, FileTreeModel treeModel,
			  MainTable table, ArchiverFrame frame) {
		JMenu file = new JMenu("File");
		JMenuItem open = new JMenuItem(new OpenAction(this));
		JMenuItem unzip = new JMenuItem(new UnzipAction(table, this));
		JMenuItem exit = new JMenuItem(new ExitAction());
		JMenuItem zip = new JMenuItem(new MakeZipAction(this));
		file.add(open);
		file.add(unzip);
		file.add(zip);
		file.addSeparator();
		file.add(exit);
		JMenu view = new JMenu("View");
		JMenu columns = new JMenu("Visible columns");
		view.add(columns);
		JCheckBoxMenuItem name = new JCheckBoxMenuItem("Name", true);		
		name.addItemListener(new SelectionListener(model, 0));
		columns.add(name);		
		JCheckBoxMenuItem path = new JCheckBoxMenuItem("Full path", true);
		path.addItemListener(new SelectionListener(model, 1));
		columns.add(path);		
		JCheckBoxMenuItem comment = new JCheckBoxMenuItem("Comment", true);
		comment.addItemListener(new SelectionListener(model, 2));
		columns.add(comment);
		JCheckBoxMenuItem date = new JCheckBoxMenuItem("Modification date", true);
		date.addItemListener(new SelectionListener(model, 3));
		columns.add(date);
		JCheckBoxMenuItem size = new JCheckBoxMenuItem("Size", true);
		size.addItemListener(new SelectionListener(model, 4));
		columns.add(size);
		JMenuBar mainMenu = new JMenuBar();
		mainMenu.add(file);
		mainMenu.add(view);
		return mainMenu;
	}
	
	private JToolBar createToolBar(MainTableModel model, FileTreeModel treeModel,
			  MainTable table) {
		JToolBar tools = new JToolBar();
		tools.add(new OpenAction(this));
		tools.add(new UnzipAction(table, this));
		tools.add(new MakeZipAction(this));
		return tools;
	}
	
	public void loadArchive(IDirectory directory) {
		myModel.setDirectory(directory);
		myModel.setArchive(directory);
		myTreeModel.setDirectory(directory);
	}
}

