package ru.amse.timkina.archiver.ui.menu;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;


import ru.amse.timkina.archiver.zipprocessor.ZipMaker;
import ru.amse.timkina.archiver.filetree.IDirectory;
import ru.amse.timkina.archiver.ui.ArchiverFrame;

public class MakeZipAction extends AbstractAction {
	
	private File myCurrentDir;
	private final ArchiverFrame myFrame;	
	
	public MakeZipAction(ArchiverFrame frame) {
		putValue(NAME, "Pack");
		putValue(SHORT_DESCRIPTION, "Create zip archive");
		myFrame = frame;
		URL zipURL = myFrame.getClass().getResource("Zip.jpg");	
		ImageIcon makeZip = new ImageIcon(zipURL);
		putValue(SMALL_ICON, makeZip);
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		JFileChooser fcFiles = new JFileChooser();
		fcFiles.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		fcFiles.setDialogTitle("Files to zip");
		fcFiles.setCurrentDirectory(myCurrentDir);
		fcFiles.setMultiSelectionEnabled(true);
		int res = fcFiles.showOpenDialog(fcFiles);
		if (res == JFileChooser.APPROVE_OPTION) {
			myCurrentDir = fcFiles.getCurrentDirectory();
			final File[] selectedFiles = fcFiles.getSelectedFiles();
			JFileChooser fc = new JFileChooser();
		    fc.setCurrentDirectory(myCurrentDir);
		    fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
		    fc.setDialogTitle("Where to zip");
		    int r = fc.showOpenDialog(fc);		
		    if (r == JFileChooser.APPROVE_OPTION) {
			    final File result = fc.getSelectedFile();
            	final ZipMaker zip = new ZipMaker();
            	final JDialog dialog = new JDialog(myFrame, "Wait, please", false);
    		    dialog.setLocationRelativeTo(myFrame);
    		    dialog.setSize(200, 150);
    		    dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
    		    JLabel label = new JLabel("Zipping", JLabel.CENTER);
    		    label.setOpaque(true);
    		    dialog.add(label, BorderLayout.CENTER);	    	
    		    dialog.setVisible(true);
    		    dialog.repaint();
    		    SwingUtilities.invokeLater(new Runnable() {		    	
    		    	@Override
    		    	public void run() {
    		    		try {    	
    		    			IDirectory newDirectory = zip.makeZip(result, selectedFiles);
    		    			myFrame.loadArchive(newDirectory);
    		    			dialog.dispose();
    		    		} catch (IOException ex) {
    		    			JOptionPane.showMessageDialog(myFrame, ex.getMessage());
    		    		}
    		    	}
    		    });			   
		    }
		}		
	}
}
