package ru.amse.timkina.archiver.ui.table;


import java.awt.event.*;
import java.util.ArrayList;
import java.util.List;
import javax.swing.*;
import javax.swing.table.*;

import ru.amse.timkina.archiver.filetree.IDirectory;
import ru.amse.timkina.archiver.filetree.IFile;
import ru.amse.timkina.archiver.ui.DirectoryChangeListener;

public class MainTable extends JTable {
	private final List<DirectoryChangeListener> myListeners = new ArrayList<DirectoryChangeListener>();
	
	public MainTable(MainTableModel model) {
		super(model);
		final MainTableModel mainModel = model;
		addMouseListener(new MyListener(mainModel));	
		setRowSorter(new TableRowSorter<TableModel>(getModel()) {
            @Override
            protected boolean useToString(int column) {
                return false;
            }
        });	
	}
	
	public void removeDirectoryChangeListener(DirectoryChangeListener l) {
		myListeners.remove(l);
	}
	
	public void addDirectoryChangeListener(DirectoryChangeListener l) {
		myListeners.add(l);
	}
	
	private class MyListener extends MouseAdapter {
		private final MainTableModel myModel;
		
		public MyListener(MainTableModel mainModel) {
			myModel = mainModel;
		}
		public void mouseClicked(MouseEvent e) {
			if (e.getClickCount() == 2) {
				int rowNum = convertRowIndexToModel(rowAtPoint(e.getPoint()));	
				IDirectory directory = myModel.getDirectory();
			    if (rowNum > 0) {				    	
				    if (rowNum < directory.getDirectories().size()) {
				    	IFile newDirectory = directory.getDirectories().get(rowNum);
			    		IDirectory newDirectoryAsDirectory = (IDirectory) newDirectory;
						(newDirectoryAsDirectory).addUp();
				    	myModel.setDirectory(newDirectoryAsDirectory);
				    	for (DirectoryChangeListener l : myListeners) {
							l.directoryChanged(newDirectoryAsDirectory, false, directory);
						}
				    }			
			    } else {
			    	IDirectory directoryParent = directory.getParent();
					myModel.setDirectory(directoryParent);
			    	for (DirectoryChangeListener l : myListeners) {
						l.directoryChanged(directoryParent, true, directory);
					}	
			    }
			}		
		}
	}
}
