package ru.amse.timkina.archiver.zipprocessor;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Collections;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.zip.*;

import ru.amse.timkina.archiver.filetree.IFile;
import ru.amse.timkina.archiver.filetree.IDirectory;

public class Unzipper {
	private List<IFile> myFiles  = new ArrayList<IFile>();
	
	protected List<IFile> getFiles() {
		return Collections.unmodifiableList(myFiles);
	}
	
	/*package*/ void getFromOne(IFile f, Map<IFile, String> dirMap, Map<String, String> fileMap) {		
		if (f.isDirectory()) {
			for (IFile newFile: f.getFiles()) {			
				myFiles.add(newFile);
				fileMap.put(newFile.getName(), dirMap.get(f) + '/' + newFile.getShortName());
			}
			for (IDirectory d : f.getDirectories()) {				
				dirMap.put(d,dirMap.get(f) + '/' + d.getShortName());								
				getFromOne(d, dirMap, fileMap);
			}
		} else {
			myFiles.add(f);
			fileMap.put(f.getName(), f.getName());		
		}		
	}
	
	/*package*/ void getAll(List<IFile> files, Map<IFile, String> dirMap, 
			Map<String, String> fileMap) {
		myFiles = new ArrayList<IFile>();		
		for (IFile f : files) {
			getFromOne(f, dirMap, fileMap);
		}	
	}
	
	public void unzip(String zipFileName, List<IFile> files, String unzipFilePath) throws IOException {
		Map<String, String> fileMap = new HashMap<String, String>();
		Map<IFile, String> dirMap = new HashMap<IFile, String>();
		fillFileMap(files, fileMap, dirMap);	
		fillDirMap(files, fileMap);	
		ZipInputStream reader = new ZipInputStream(new FileInputStream(zipFileName));
		try {
			extractArchive(unzipFilePath, fileMap, reader);		
		} finally {
			reader.close();
		}
	}

	private void extractArchive(String unzipFilePath,
			Map<String, String> fileMap, ZipInputStream reader)
			throws IOException, FileNotFoundException {
		ZipEntry entry;
		while ((entry = reader.getNextEntry()) != null) {
			if (fileMap.containsKey(entry.getName())) {		       
				String name = fileMap.get(entry.getName());
				String path = buidPath(name);
			    File forDirs = new File(unzipFilePath + File.separator + path);
			    File real =  new File(unzipFilePath + '/' + name);
			    forDirs.mkdirs();
			    FileOutputStream writer = new FileOutputStream(real);
			    byte[] buf = new byte[1024];
			    int number;
			    try {
			        while ((number = reader.read(buf)) >= 0) {
				        writer.write(buf, 0 , number);
			        }
			    } finally {
		            writer.close();	
			    }
		        real.setLastModified(entry.getTime());		            
			}
		}
	}

	private String buidPath(String name) {
		String path;			
		if (name.lastIndexOf('/') != -1) {
		    path = name.substring(0, name.lastIndexOf('/'));
		} else {
		    path = "";
		}
		return path;
	}

	private void fillDirMap(List<IFile> files, Map<String, String> fileMap) {
		for (IFile f : files) {
			if (!f.isDirectory()) {
				fileMap.put(f.getName(), f.getShortName());
			}
		}
	}

	private void fillFileMap(List<IFile> files, Map<String, String> fileMap,
			Map<IFile, String> dirMap) {
		for (IFile f : files) {
			if (f.isDirectory()) {
				dirMap.put(f, f.getShortName());
			}
		}	
		getAll(files, dirMap, fileMap);
	}
}
